<?php
/**
* @package: phpBB 3.0.5 :: MSSTI Sitemap -> root/sitemap.php
* @version: $Id: rss.php, v 1.2.1 2009/06/01 09:06:01 leviatan21 Exp $
* @copyright: (c) 2008-2009 leviatan21 < info@mssti.com > (Gabriel) http://www.mssti.com/phpbb3/
* @license: http://opensource.org/licenses/gpl-license.php GNU Public License
* @author: leviatan21 - http://www.phpbb.com/community/memberlist.php?mode=viewprofile&u=345763
*
**/

/**
* @ignore
*	http://www.sitemaps.org/protocol.php
* 	http://www.sitemappro.com/examples.html
*	http://www.scottandrew.com/weblog/articles/dom_espanol
*******************************************************************************************************************************************************************************************************
*		<urlset>
*			required
*			Encapsulates the file and references the current protocol standard.
*
*		<url>
*			required
*			Parent tag for each URL entry. The remaining tags are children of this tag.
*
*		<loc>
*			required
*			URL of the page. This URL must begin with the protocol (such as http) and end with a trailing slash, if your web server requires it. This value must be less than 2,048 characters.
*
*		<lastmod>
*			optional
*			The date of last modification of the file. This date should be in W3C Datetime format. This format allows you to omit the time portion, if desired, and use YYYY-MM-DD.
*			Note that this tag is separate from the If-Modified-Since (304) header the server can return, and search engines may use the information from both sources differently.
*
*		<changefreq>
*			optional
*			How frequently the page is likely to change. This value provides general information to search engines and may not correlate exactly to how often they crawl the page. Valid values are:
*				always
*				hourly
*				daily
*				weekly
*				monthly
*				yearly
*				never
*			The value "always" should be used to describe documents that change each time they are accessed. The value "never" should be used to describe archived URLs.
*			Please note that the value of this tag is considered a hint and not a command. Even though search engine crawlers may consider this information when making decisions, 
*			they may crawl pages marked "hourly" less frequently than that, and they may crawl pages marked "yearly" more frequently than that. 
*			Crawlers may periodically crawl pages marked "never" so that they can handle unexpected changes to those pages.
*
*		<priority>
*			optional
*			The priority of this URL relative to other URLs on your site. Valid values range from 0.0 to 1.0. 
*			This value does not affect how your pages are compared to pages on other sites—it only lets the search engines know which pages you deem most important for the crawlers.
*			The default priority of a page is 0.5.
*			Please note that the priority you assign to a page is not likely to influence the position of your URLs in a search engine's result pages. 
*			Search engines may use this information when selecting between URLs on the same site, so you can use this tag to increase the likelihood that your most important pages are present in a search index.
*			Also, please note that assigning a high priority to all of the URLs on your site is not likely to help you. Since the priority is relative, it is only used to select between URLs on your site.
*
*	Entity escaping
* 		Your Sitemap file must be UTF-8 encoded (you can generally do this when you save the file). 
* 		As with all XML files, any data values (including URLs) must use entity escape codes for the characters listed in the function sitemap_content()
* 		In addition, all URLs (including the URL of your Sitemap) must be URL-escaped and encoded for readability by the web server on which they are located. 
* 		However, if you are using any sort of script, tool, or log file to generate your URLs (anything except typing them in by hand), this is usually already done for you. 
* 		Please check to make sure that your URLs follow the  RFC-3986 standard for URIs, the RFC-3987  standard for IRIs, and the XML standard.
**/

// Connect
define('IN_PHPBB', true);
$phpbb_root_path = (defined('PHPBB_ROOT_PATH')) ? PHPBB_ROOT_PATH : './';
$phpEx = substr(strrchr(__FILE__, '.'), 1);
include($phpbb_root_path . 'common.' . $phpEx);

// Start session management
$user->session_begin();
$auth->acl($user->data);
$user->setup(array('acp/forums', 'viewforum'));

// Give us some basic information
sitemap_init();

// $forum_id	= request_var('id', 0);

// Set a browser
$format = strtolower(request_var('format', sitemap_format(), true));

// Run the sql and fill the data
$rowset	= sitemap_branch();

// Okay, lets dump out the page ...
if (sizeof($rowset))
{
	if ($format == 'xml')
	{
		// Send Header
		header("Content-Type: text/xml; charset=UTF-8");

		foreach ($rowset as $row)
		{
			$template->assign_block_vars('topics', array(
				'T_LINK'		=> $row['loc'],
				'LINK_LASTMOD'	=> $row['lastmod'],
				'LINK_FREQ'		=> $row['frequently'],
				'LINK_PRIO'		=> $row['priority'],
			));
		}

		$template->assign_vars(array(
			'S_XML_SITEMAP'	=> true,
			'ENCODING'		=> '<?xml version="1.0" encoding="UTF-8"?>',
			'U_INDEX'		=> URL_INDEX,
			'INDEX_LASTMOD'	=> date("Y-m-d\TH:i:sP", time()),
			'INDEX_FREQ'	=> FREQ_INDEX,
			'INDEX_PRIO'	=> PRIO_INDEX,
		));

		$template->set_filenames(array(
			'body' => 'sitemap_body.html')
		);

		$template->display('body');

		garbage_collection();
		exit_handler();
	}
	else if ($format == 'list')
	{
		// application/xhtml+xml not used because of IE
		header("Content-type: text/html; charset=UTF-8");

		foreach ($rowset as $row)
		{
			$template->assign_block_vars('topics', array(
				'T_LINK'	=> $row['loc'] . "\r\n",
			));
		}

		$template->assign_vars(array(
			'S_LIST_SITEMAP'	=> true,
			'U_INDEX'			=> URL_INDEX . "\r\n",
		));

		$template->set_filenames(array(
			'body' => 'sitemap_body.html')
		);

		$template->display('body');

		garbage_collection();
		exit_handler();
	}
	else
	{
		$template->assign_vars(array(
			'S_HTML_SITEMAP'	=> true,
			'U_BBCODE_GUIDE'	=> append_sid("{$phpbb_root_path}faq.$phpEx", 'mode=bbcode'),
			'U_TERMS_USE'		=> append_sid("{$phpbb_root_path}ucp.$phpEx", 'mode=terms'),
			'U_PRIVACY'			=> append_sid("{$phpbb_root_path}ucp.$phpEx", 'mode=privacy'),
		));

		make_tree();

		foreach ($rowset as $row )
		{
			if ($row['type'] == TYPE_TOPIC)
			{
				$template->assign_block_vars('jumpbox_forums.topics', array(
					'TOPIC_FORUM_ID'	=> $row['forum_id'],
					'TOPIC_NAME'		=> html_entity_decode($row['name']),
					'U_TOPIC'			=> $row['loc'],
				));
			}
		}

		// Output page
		page_header($user->lang['FORUM_SITEMAP']);

		$template->set_filenames(array(
			'body' => 'sitemap_body.html')
		);

		page_footer();
	}
}

unset($rowset);

/******************************************************************************************************************************************************************************************************
* 
******************************************************************************************************************************************************************************************************/

/**
* Define initial vars
**/
function sitemap_init()
{
	global $phpEx;

	define('URL_BOARD',			generate_board_url() . '/');
	define('URL_INDEX',			URL_BOARD . "index.$phpEx");
	define('URL_VIEWCAT',		URL_BOARD . "viewforum.$phpEx");
	define('URL_VIEWFORUM',		URL_BOARD . "viewforum.$phpEx");
	define('URL_VIEWTOPIC',		URL_BOARD . "viewtopic.$phpEx");

	define('FREQ_INDEX',		'monthly');
	define('FREQ_CATERORY',		'weekly');
	define('FREQ_FORUM',		'weekly');
	define('FREQ_TOPIC',		'daily');

	define('PRIO_INDEX',		'1.0');
	define('PRIO_CATERORY',		'0.2');
	define('PRIO_FORUM',		'0.3');
	define('PRIO_TOPIC',		'0.5');

	define('TYPE_CAT',			1);
	define('TYPE_FORUM',		2);
	define('TYPE_TOPIC',		3);

}

/**
* Enter description here...
*
* @param unknown_type $action
* @param unknown_type $forum_id
* @param unknown_type $select_all
* @param unknown_type $acl_list
* @param unknown_type $force_display
**/
function make_tree($forum_id = false, $select_all = false, $acl_list = false, $force_display = false)
{
	global $config, $auth, $template, $user, $db;

//	// We only return if the jumpbox is not forced to be displayed (in case it is needed for functionality)
//	if (!$config['load_jumpbox'] && $force_display === false)
//	{
//		return;
//	}

	$sql = 'SELECT forum_id, forum_name, forum_status, parent_id, forum_type, left_id, right_id, forum_topics 
		FROM ' . FORUMS_TABLE . '
		ORDER BY left_id ASC';
	$result = $db->sql_query($sql, 600);

	$right = $padding = 0;
	$padding_store = array('0' => 0);
	$display_jumpbox = false;
	$iteration = 0;

	// Sometimes it could happen that forums will be displayed here not be displayed within the index page
	// This is the result of forums not displayed at index, having list permissions and a parent of a forum with no permissions.
	// If this happens, the padding could be "broken"

	while ($row = $db->sql_fetchrow($result))
	{
		if ($row['left_id'] < $right)
		{
			$padding++;
			$padding_store[$row['parent_id']] = $padding;
		}
		else if ($row['left_id'] > $right + 1)
		{
			// Ok, if the $padding_store for this parent is empty there is something wrong. For now we will skip over it.
			// @todo digging deep to find out "how" this can happen.
			$padding = (isset($padding_store[$row['parent_id']])) ? $padding_store[$row['parent_id']] : $padding;
		}

		$right = $row['right_id'];

		if ($row['forum_type'] == FORUM_CAT && ($row['left_id'] + 1 == $row['right_id']))
		{
			// Non-postable forum with no subforums, don't display
			continue;
		}

		if (!$auth->acl_get('f_list', $row['forum_id']))
		{
			// if the user does not have permissions to list this forum skip
			continue;
		}

		if ($acl_list && !$auth->acl_gets($acl_list, $row['forum_id']))
		{
			continue;
		}

		if (!$display_jumpbox)
		{
//			$template->assign_block_vars('jumpbox_forums', array(
//				'FORUM_ID'		=> ($select_all) ? 0 : -1,
//				'FORUM_NAME'	=> ($select_all) ? $user->lang['ALL_FORUMS'] : $user->lang['SELECT_FORUM'],
//				'S_FORUM_COUNT'	=> $iteration)
//			);

			$iteration++;
			$display_jumpbox = true;
		}

		$template->assign_block_vars('jumpbox_forums', array(
			'U_LINK'		=> sitemap_content(URL_VIEWFORUM . '?f=' . $row['forum_id']), 
			'FORUM_ID'		=> $row['forum_id'],
			'FORUM_NAME'	=> $row['forum_name'],
			'FORUM_CLOSED'	=> $row['forum_status'],
			'FORUM_TITLE'	=> ($row['forum_status']) ? $user->lang['FORUM_LOCKED'] : $user->lang['VIEW_FORUM'],
			'FORUM_POST'	=> ($row['forum_topics']) ? true : false,
			'S_FORUM_COUNT'	=> $iteration,
			'S_IS_CAT'		=> ($row['forum_type'] == FORUM_CAT) ? true : false,
			'S_IS_LINK'		=> ($row['forum_type'] == FORUM_LINK) ? true : false,
			'S_IS_POST'		=> ($row['forum_type'] == FORUM_POST) ? true : false)
		);

		if ($row['forum_topics'])
		{
			$topic_result = sitemap_get_topics($row['forum_id']);
			while ($topic_rows = $db->sql_fetchrow($topic_result))
			{
				$template->assign_block_vars('jumpbox_forums.topics', array(
					'U_LINK'		=> sitemap_content(URL_VIEWTOPIC . '?f=' . $row['forum_id'] . '&t=' . $topic_rows['topic_id']), 
					'FORUM_ID'		=> $row['forum_id'],
					'TOPIC_ID'		=> $topic_rows['topic_id'],
					'TOPIC_NAME'	=> $topic_rows['topic_title'],
					'TOPIC_CLOSED'	=> $topic_rows['topic_status'],
					'TOPIC_TITLE'	=> ($topic_rows['topic_status']) ? $user->lang['TOPIC_LOCKED_SHORT'] : $user->lang['VIEW_TOPIC'],
				));

				for ($i = 0; $i < $padding; $i++)
				{
					$template->assign_block_vars('jumpbox_forums.topics.levels', array());
				}
			}
			$db->sql_freeresult($topic_result);
		}

		for ($i = 0; $i < $padding; $i++)
		{
			$template->assign_block_vars('jumpbox_forums.level', array());
		}
		$iteration++;
	}
	$db->sql_freeresult($result);
	unset($padding_store);

	$length = (int) strlen($user->lang['TOPICS'] . '&raquo; ');
	$extra_space = '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; ';
	for ($i = 0; $i < $length; $i++)
	{
		$extra_space .='&nbsp;';
	}
	$template->assign_vars(array(
		'EXTRA_SPACE'		=> $extra_space,
		'S_DISPLAY_JUMPBOX'	=> $display_jumpbox,
	));

	return;
}

/**
* Get forum branch...
*
* @return unknown
**/
function sitemap_branch()
{
	global $db, $auth;
	$rowset		= array();

	$categories_result = sitemap_get_categories();

	// Get Categories
	while ($categories_row = $db->sql_fetchrow($categories_result))
	{
		// Exclude forums the user is not able to read
		if (!$auth->acl_get('f_list', (int) $categories_row['forum_id']))
		{
			continue;
		}
		// Exclude passworded forum
		if ($categories_row['forum_password'] != '')
		{
			continue;
		}

//		$rowset[] = array(	'category_id'	=> $categories_row['forum_id'],
		$rowset[$categories_row['forum_id']] = array(
							'category_id'	=> $categories_row['forum_id'],
							'type'			=> TYPE_CAT,
							'loc'			=> sitemap_content(URL_VIEWCAT . '?f=' . $categories_row['forum_id']), 
							'name'			=> $categories_row['forum_name'],
		//					'lastmod'		=> date("Y-m-d\TH:i:sP", $categories_row['forum_last_post_time']),
							'lastmod'		=> ($categories_row['forum_last_post_time']) ? date("Y-m-d\TH:i:sP", $categories_row['forum_last_post_time']) : date("Y-m-d\TH:i:sP", time()),
							'frequently'	=> FREQ_CATERORY,
							'priority'		=> PRIO_CATERORY,
		);

		// Get Forums
		$forum_result = sitemap_get_forum($categories_row['forum_id'], $categories_row['left_id'], $categories_row['right_id']);
		while ($forum_rows = $db->sql_fetchrow($forum_result))
		{
			// Exclude forums the user is not able to read
			if (!$auth->acl_get('f_list', (int) $forum_rows['forum_id']))
			{
				continue;
			}
			// Exclude passworded forum
			if ($forum_rows['forum_password'] != '')
			{
				continue;
			}
			// Skip same id as before
			if ( (int) $categories_row['forum_id'] == (int) $forum_rows['forum_id'])
			{
				continue;
			}
			/*
			if ($forum_rows['forum_type'] == FORUM_LINK)
			{
				continue;
			}
			*/
			($forum_rows['forum_last_post_time']) ? $rowset[$categories_row['forum_id']]['lastmod'] = date("Y-m-d\TH:i:sP", $forum_rows['forum_last_post_time']) : false;
//			$rowset[] = array(	'category_id'	=> $categories_row['forum_id'],
			$rowset[$forum_rows['forum_id']] = array(
								'category_id'	=> $categories_row['forum_id'],
								'forum_id'		=> $forum_rows['forum_id'],
								'type'			=> TYPE_FORUM,
								'loc'			=> sitemap_content(URL_VIEWCAT . '?f=' . $forum_rows['forum_id']), 
								'name'			=> $forum_rows['forum_name'],
			//					'lastmod'		=> date("Y-m-d\TH:i:sP", $forum_rows['forum_last_post_time']),
								'lastmod'		=> ($forum_rows['forum_last_post_time']) ? date("Y-m-d\TH:i:sP", $forum_rows['forum_last_post_time']) : date("Y-m-d\TH:i:sP", time()) ,
								'frequently'	=> FREQ_FORUM,
								'priority'		=> PRIO_FORUM,
			);

			// Get Topics
			$topic_result = sitemap_get_topics($forum_rows['forum_id']);
			while ($topic_rows = $db->sql_fetchrow($topic_result))
			{
				($topic_rows['topic_last_post_time']) ? $rowset[$categories_row['forum_id']]['lastmod'] = date("Y-m-d\TH:i:sP", $topic_rows['topic_last_post_time']) : false;
				$rowset[] = array(	'category_id'	=> $categories_row['forum_id'],
									'forum_id'		=> $forum_rows['forum_id'],
									'topic_id'		=> $topic_rows['topic_id'],
									'type'			=> TYPE_TOPIC,
									'loc'			=> sitemap_content(URL_VIEWTOPIC . '?f=' . $topic_rows['forum_id'] . '&t=' . $topic_rows['topic_id']), 
									'name'			=> $topic_rows['topic_title'],
									'lastmod'		=> date("Y-m-d\TH:i:sP", $topic_rows['topic_last_post_time']),
									'frequently'	=> FREQ_TOPIC,
									'priority'		=> PRIO_TOPIC,
				);
			}
			$db->sql_freeresult($topic_result);
		}
		$db->sql_freeresult($forum_result);
	}
	$db->sql_freeresult($categories_result);

	return $rowset;
}

/**
* Your Sitemap file must be UTF-8 encoded 
* As with all XML files, any data values (including URLs) must use entity escape codes for the characters listed in the table below.
*
* @param string		$content => link or name
* @return string	UTF-8 encoded
**/
function sitemap_content($content)
{
	$content = str_replace(
	array(
			'&',		// Ampersand		&	&amp;
			"'",		// Single Quote		'	&apos;
			'"',		// Double Quote		"	&quot;
			'>',		// Greater Than		>	&gt;
			'<',		// Less Than		<	&lt;
	),
	array(
			'&amp;',	// Ampersand		&	&amp;
			'&apos;',	// Single Quote		'	&apos;
			'&quot;',	// Double Quote		"	&quot;
			'&gt;',		// Greater Than		>	&gt;
			'&lt;',		// Less Than		<	&lt;
	), $content);

	// Other control characters
	$content = preg_replace('#(?:[\x00-\x1F\x7F]+|(?:\xC2[\x80-\x9F])+)#', '', $content);

	return $content;
}

/**
* Get a list of categories forums...
*
* @return array		with categories data
**/
function sitemap_get_categories()
{
	global $db;

	// Send Categories
	$categories_sql = "SELECT f.forum_id, f.forum_name, f.forum_password, f.forum_last_post_time, f.parent_id, f.left_id, f.right_id 
				FROM " . FORUMS_TABLE ." f 
				WHERE f.forum_type = " . FORUM_CAT . "
				ORDER BY f.left_id ASC";
	return $db->sql_query($categories_sql);	
}

/**
* Get a list of forums...
*
* @param array		with categories
* @return array		with forums data
**/
function sitemap_get_forum($forum_id, $left_id, $right_id)
{
	global $db;

	$forum_sql = "SELECT sf.forum_id, sf.forum_name, sf.forum_status, sf.forum_last_post_time, sf.forum_password 
				FROM " . FORUMS_TABLE . " f, " . FORUMS_TABLE . " sf 
				WHERE f.forum_id = " . (int) $forum_id . " AND sf.forum_type <> " . FORUM_LINK . "
					AND ( sf.left_id BETWEEN " . (int) $left_id . " AND " . (int) $right_id . " ) 
				ORDER BY sf.left_id ASC";
	return $db->sql_query($forum_sql);
}

/**
* Get a list of topics...
*
* @param array		with forums
* @return array		with topics data
**/
function sitemap_get_topics($forum_id)
{
	global $db;

	$topic_sql = "SELECT t.forum_id, t.topic_id, t.topic_status, t.topic_title, t.topic_last_post_time 
				FROM " . TOPICS_TABLE . " t 
				WHERE t.topic_approved=1 AND t.forum_id = " . (int) $forum_id . '
				ORDER BY t.topic_type DESC, t.topic_last_post_time DESC';
	return $db->sql_query($topic_sql);
}

/**
* Get a browser...
*
* @return string
**/
function sitemap_format()
{
	$user_agent = (!empty($_SERVER['HTTP_USER_AGENT'])) ? htmlspecialchars((string) $_SERVER['HTTP_USER_AGENT']) : '';

	if (strpos(strtolower($user_agent), 'google') !== false)
	{
		return "xml";
	}
	if (strpos(strtolower($user_agent), 'yahoo') !== false)
	{
		return "list";
	}
	return strtolower($user_agent);
}

?>